function msg = check_rcsf (handle, rows, cols, slices, frames)
%  CHECK_RCSF  determine validity of row,col,slice,frame lists (internal use)
%
%      msg = check_rcsf (handle, rows, cols, slices, frames)
%
%  examines the lists of rows, cols, slices, & frames, compares them to the 
%  properties of the MINC file specified by handle, and generates
%  a reasonably useful error message if there's any inconsistency.
%  check_rcsf is meant to be called by other EMMA functions, particularly
%  getimages and putimages.
%
%  The specific conditions that cause an error message are:
%
%     - the file has no time dimension, but a frame list was given
%     - the file has a time dimension, but no frame list was given
%     - the file has no slice dimension, but a slice list was given
%     - the file has a slice dimension, but no slice list was given
%     - there were out-of-range frames: frame number either greater than
%       the number of frames or less than one
%     - there were out-of-range slices
%
%  New error conditions are:
%
%     - there were out-of-range rows
%     - there were out-of-range columns
%
%  Conditions which used to be disallowed but are now allowed are:
%
%     - both slices and frames have multiple values
%
%  If there are no problems, then the empty matrix is returned.
%
% (DERIVED FROM CHECK_SF.M IN ORIGINAL EMMA)

msg = [];

% First retrieve the number of frames and slices from the global workspace

eval(['global DimSizes' int2str(handle)]);
dim_sizes = eval(['DimSizes' int2str(handle)]);
num_frames = dim_sizes(1);
num_slices = dim_sizes(2);
num_rows   = dim_sizes(3);
num_cols   = dim_sizes(4);

if (num_frames == 0)
   if ~isempty (frames)
      msg = 'Image has no time dimension: list of frames not allowed';
   end
end

if (isempty (frames)) & (num_frames > 0)
   msg = 'Image has a time dimension; you must specify frames';
end

if (num_slices == 0)
   if ~isempty (slices)
      msg = 'Image has no slice dimension: list of slices not allowed';
   end
end

if (isempty (slices)) & (num_slices > 0)
   msg = 'Image has a slice dimension; you must specify slices';
end

if (find (slices > num_slices | slices <= 0))
   msg = 'Out-of-range slice number given';
end

if (find (frames > num_frames | frames <= 0))
   msg = 'Out-of-range frame number given';
end

if (find (rows > num_rows | rows <= 0))
   msg = 'Out-of-range row number given';
end

if (find (cols > num_cols | cols <= 0))
   msg = 'Out-of-range column number given';
end

