%GETVOLUMES  Retrieve whole or partial volumes from an open MINC file.
%
%  volumes = getvolumes (handle , rows, cols, slices, frames)
%
%  reads whole or partial volumes from the MINC file specified by
%  handle.  
%
%  the output `volumes' is (generally) a four dimensional array
%  with dimensions (row, column, slice, frame)
%
%  NOTES:
%
%  Vectors of indices will be sorted to ensure that they are in
%  ascending order.
%
%  Some files don't have slice and/or frame dimensions,
%  leading to the following possible outputs:
%
%  1)  file has row, col, slice but no frame:
%
%         3D matrix -> (rows, cols, slices)
%
%  2)  file has row, col, frame but no slice:
%
%         4D matrix with one singleton dim -> (rows, cols, 1, slices)
%
%  3)  file has row, col, but no slice or frame:
%
%         2D matrix -> (rows, cols)
%  
%  also, instead of `time' some files have alternate frame dimensions 
%  such as echo_time, (heart) phase, (future) inversion_time...
%  this could necessitate indexing such as the following:
%
%    (rows, cols, slices, echo_times, frames)
%
%  and on top of that, the occasional file might have the nerve to be
%  vector valued, requiring indexing like
%
%    (components, rows, cols, slices, frames)
%
%  the most general indexing might therefore look like this:
%
%    (components, rows, cols, slices, loop1, loop2)
%
%  where loop1 might be echo times in a multi-echo acq that is
%  serially repeated in loop2
%
%  EXAMPLES:
%
%  get mid-sagittal slice from transverse dataset (64 x 64 x 20 slices):
%
%  slice = getvolumes(handle, [1:64], 32, [1:20]);

% ------------------------------ MNI Header ----------------------------------
%@NAME       : getvolumes
%@INPUT      : handle - tells which MINC file (or internal-to-MATLAB
%              image data) to read from
%              rows - list of rows (1-based) to read
%              cols - list of columns (1-based) to read
%              slices - list of slices (1-based) to read
%              frames - list of frames (1-based) to read
%@OUTPUT     : 
%@RETURNS    : volumes - multidimensional matrix with volume/series data
%
%@DESCRIPTION: Reads images from the MINC file associated with a MATLAB
%              image handle.  The handle must have an associated MINC file;
%              purely internal image sets are not yet supported.  
%
%              Note that if care is not taken, this can easily take up
%              large amounts of memory.  Each image takes up 128 k of
%              MATLAB memory, so reading all frames for a single slice
%              from a 21-frame dynamic study will take up 2,688 k.
%              When various analysis routines are carried out on this
%              data, the amount of memory allocated by MATLAB can
%              easily triple or quadruple.  getimages attempts to combat
%              this by assigning a "maximum" number of images for each
%              of PET's four main SGI's (as of 93/7/6: priam, duncan, lear,
%              portia) depending on their current memory 
%              configurations.  If the number of slices/frames specified
%              is greater than this "maximum", getimages will print a 
%              warning and then read the data.  
%
%              Note added 8 years later by rhoge:
%              Most of our computers have 1 Gigabyte of RAM these days,
%              so in this version the memory restrictions have been greatly
%              relaxed

function volumes = getvolumes (...
    handle, ...
    rows, ...
    cols, ...
    slices, ...
    frames ...
    )

%
%@METHOD     : 
%@GLOBALS    : Filename#
%@CALLS      : check_rcsf to check validity of rows/cols/slices/frames args
%              mireadvolumes (CMEX)
%@CREATED    : June 1993, Greg Ward & Mark Wolforth
%@MODIFIED   : 6 July 1993, Greg Ward: 
%             30 May 1994, Greg Ward: added start_row and num_rows, 
%                          completely rewrote help section
%@BASTARDIZED : 16 Nov 2001, Rick Hoge.
%@VERSION    : $Id: getvolumes.m,v 1.1.1.1 2001/11/20 19:25:05 rhoge Exp $
%              $Name:  $
%-----------------------------------------------------------------------------


% Check for valid number of arguments

if (nargin < 1) | (nargin > 5)
   error ('Incorrect number of arguments.');
end

if (nargin < 2)
   rows = [];         % no rows vector given, so make it empty
end

if (nargin < 3)         % no cols vector given, so make it empty
   cols = [];
end

if (nargin < 4)         % no slices vector given, so make it empty
   slices = [];
end

if (nargin < 5)         % no frames vector given, so make it empty
   frames = [];
end

% Make the handle's filename global, and check that it exists

eval(['global Filename' int2str(handle)]);
if exist (['Filename' int2str(handle)]) ~= 1
   disp ('getimages: image unknown - use openimage');
end 

% and copy it to a local variable for ease of use

filename = eval (['Filename' int2str(handle)]);

if isempty (filename)
   disp ('getimages: no MINC file associated with image, cannot read images');
end

% now make sure input arguments are valid: check_rcsf returns an error message
% if not.

s = check_rcsf (handle, rows, cols, slices, frames);
if ~isempty (s); error (s); end;

% sort the row, col, slice, frames indices to ensure ascending
% order:

rows = sort(rows);
cols = sort(cols);
slices = sort(slices);
frames = sort(frames);

% Now read the data!  (remembering to make slices and frames zero-based for
% mireadvolumes).

volumes = mireadvolumes (filename, rows-1, cols-1, slices-1, frames-1);

